@@ -219,12 +219,14 @@ nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = "next", recv_
 nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 3L, timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
-nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) serialize(x, NULL), unserialize))))
+nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) raw(1L), as.integer), class = "custom", vec = FALSE)))
 nanotest(length(nxt) == 2L)
 nanotest(is.function(nxt[[1L]]))
-nanotest(is.integer(req$send(list(rep, ctx), mode = 3L, block = 500)))
-nanotest(typeof(recv(rep, block = 500)[[1L]]) == "externalptr")
-nanotest(is.integer(send(rep, new.env(), mode = 3L, block = 500)))
+custom <- list(`class<-`(new.env(), "custom"), new.env())
+nanotest(is.integer(req$send(custom, mode = 3L, block = 500)))
+nanotest(is.integer(recv(rep, block = 500)[[1L]]))
+nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) raw(1L), as.integer), class = "custom", vec = TRUE)))
+nanotest(is.integer(send(rep, custom, mode = 3L, block = 500)))
 nanotest(identical(next_config(nxt), nxt))
 nanotest(identical(next_config(as.pairlist(c("a", "b", "c"))), nxt))
 nanotest(is.list(next_config(NULL, mark = TRUE)))
@@ -397,6 +399,7 @@ nanotest(is_nul_byte(as.raw(0L)))
 nanotest(!is_nul_byte(NULL))
 nanotest(!is_error_value(1L))
 nanotesterr(messenger("invalidURL"), "argument")
+nanotest(is.raw(md5 <- nanonext:::md5_object("secret base")) && length(md5) == 32L)
 nanotest(is.double(mclock()))
 nanotestn(msleep(1L))
 nanotestn(msleep(1))
@@ -514,15 +517,6 @@ nanotest(identical(call_aio("a"), "a"))
 nanotest(identical(call_aio_("a"), "a"))
 nanotestn(stop_aio("a"))
 
-nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")
-nanotest(length(sha256(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 32L)
-nanotest(nchar(sha512(c(1.1, 2.2), as.raw(1L))) == 128L)
-nanotest(length(sha512("testing", convert = FALSE)) == 64L)
-nanotest(length(sha384(runif(5e3L), "KEY", convert = FALSE)) == 48L)
-nanotest(nchar(sha384(7000L, key = NULL, convert = TRUE)) == 96L)
-nanotest(length(sha224(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 28L)
-nanotest(sha224(as.raw(0L)) == "fff9292b4201617bdc4d3053fce02734166a683d7d858a7f5f59b073")
-nanotest(is.character(sha256(quote(a))))
 nanotest(base64enc("test") == "dGVzdA==")
 nanotest(base64dec(base64enc("test")) == "test")
 nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
